#!/usr/bin/env python

from agatelookup.source import Source

def patch():
    """
    Patch the features of this library onto agate's core
    :class:`Table <agate.table.Table>` and
    :class:`TableSet <agate.tableset.TableSet>`.
    """
    import agate
    from agatelookup.table_lookup import TableLookup

    agate.Table.monkeypatch(TableLookup)



#!/usr/bin/env python

import agate
import agateremote
import requests
import yaml

agateremote.patch()

def default_table_url_func(root, keys, value, version=None):
    """
    Default function for converting lookup table data into a table URL.
    """
    if isinstance(keys, (list, tuple)):
        keys = '/'.join(keys)

    url = '%s/%s/%s' % (root, keys, value)

    if version:
        url += '.%s' % version

    url += '.csv'

    return url

def default_metadata_url_func(root, keys, value, version=None):
    """
    Default method of converting lookup table data into a metadata file url.
    """
    if isinstance(keys, (list, tuple)):
        keys = '/'.join(keys)

    url = '%s/%s/%s' % (root, keys, value)

    if version:
        url += '.%s' % version

    url += '.csv.yml'

    return url

def make_type_tester(meta):
    """
    Uses parsed lookup table metadata to create a :class:`.agate.TypeTester`
    that will always use correct types for the table columns. (And avoid
    the overhead of type inference.)
    """
    force = {}

    for k, v in meta['columns'].items():
        force[k] = getattr(agate, v)()

    return agate.TypeTester(force=force)

class Source(object):
    """
    A reference to an archive of lookup tables. This is a remote location with
    lookup table and metadata files at a known path structure.

    :param root:
        The root URL to prefix all data and metadata paths.
    :param table_url_func:
        A function that takes table metadata and returns an absolute URL to
        that table's data file. See :func:`default_table_url_func`.
    :param metadata_url_func:
        A function that takes table metadata and returns an absolute URL to
        that table's data file. See :func:`default_metadata_url_func`.
    :param callback:
        A function that translates the remote data table into an agate table.
        Typically this is :meth:`.agate.Table.from_csv`, but it could also be
        :meth:`.agate.Table.from_json` or a method provided by an extension.
    """
    def __init__(self, root='http://wireservice.github.io/lookup', table_url_func=default_table_url_func, metadata_url_func=default_metadata_url_func, callback=agate.Table.from_csv):
        self._root = root
        self._table_url_func = table_url_func
        self._metadata_url_func = metadata_url_func
        self._callback = callback

    def get_metadata(self, keys, value, version=None):
        """
        Fetches metadata related to a specific lookup table.

        See :meth:`Source.get_table` for parameter details.
        """
        url = self._metadata_url_func(self._root, keys, value, version)
        r = requests.get(url)

        # TKTK: handle error

        return yaml.load(r.text)

    def get_table(self, keys, value, version=None):
        """
        Fetches and creates and agate table from a specified lookup table.

        :param keys:
            Either a single string or a sequence of keys that identify the
            "left side" of the table. For example :code:`'fips'` or
            :code:`['city', 'year']`.
        :param value:
            The value that is being looked up from the given keys. For example
            :code:`'state'` or :code:`'population'`.
        :param version:
            An optional version of the given lookup, if more than one exists.
            For instance :code:`'2007'` for the 2007 edition of the NAICS codes
            or :code:`'2012'` for the 2012 version.
        """
        meta = self.get_metadata(keys, value, version)
        tester = make_type_tester(meta)

        url = self._table_url_func(self._root, keys, value, version)

        return agate.Table.from_url(url, column_types=tester, callback=self._callback)



#!/usr/bin/env python

"""
This module contains the Lookup extension to :class:`Table <agate.table.Table>`.
"""

import agateremote

from agatelookup.source import Source

agateremote.patch()

DEFAULT_SOURCE = Source()

class TableLookup(object):
    def lookup(self, source_keys, value, table_keys=None, version=None, source=None, require_match=False):
        """
        Fetch a lookup table from the remote source, matches it this table by
        its key columns, appends the value column and returns a new table
        instance.

        :param source_keys:
            A column name or a sequence of such names to match in the lookup
            table. For example :code:`'naics'` or :code:`['city', 'year']`.
        :param value:
            The value that is being looked up. For example :code:`'description'`
            or :code:`'population'`. This is the column that will be appended.
        :param table_keys:
            A column name or a sequence of such names to match in this table.
            This defaults the same values specified for `source_keys`, so it
            only needs to be specified if the column names in this table don't
            match.
        :param version:
            An optional version of the lookup to use, if more than one exists.
            For instance :code:`'2007'` for the 2007 edition of the NAICS codes
            or :code:`'2012'` for the 2012 version.
        :param source:
            An instance of :class:`.Source` that defines where lookup tables
            are located. If not specified a default source will be used that
            points to the
            `wireservice/lookup <https://github.com/wireservice/lookup>`_
            repository.
        :param require_match:
            If :code:`True`, an exception will be raised if there is a value in
            this table with no matching entry in the lookup table.
        """
        if source is None:
            source = DEFAULT_SOURCE

        table = source.get_table(source_keys, value, version)

        return self.join(table, table_keys or source_keys, source_keys, require_match=require_match)



#!/usr/bin/env python

from setuptools import setup

install_requires = [
    'agate>=1.3.0',
    'agate-remote>=0.1.1',
    'requests>=2.9.1',
    'pyyaml>=3.11'
]

setup(
    name='agate-lookup',
    version='0.1.0',
    description='agate-lookup adds remote lookup tables to agate.',
    long_description=open('README.rst').read(),
    author='Christopher Groskopf',
    author_email='chrisgroskopf@gmail.com',
    url='http://agate-lookup.readthedocs.org/',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Multimedia :: Graphics',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Scientific/Engineering :: Visualization',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages=[
        'agatelookup'
    ],
    install_requires=install_requires
)
